@@ -1,26 +1,22 @@
-// Copyright (c) 2016 The Bitcoin developers
+// Copyright (c) 2016-2018 The Bitcoin Core developers
 // Distributed under the MIT software license, see the accompanying
 // file COPYING or http://www.opensource.org/licenses/mit-license.php.
 
 #ifndef BITCOIN_POLICY_RBF_H
 #define BITCOIN_POLICY_RBF_H
 
-#include "txmempool.h"
+#include <txmempool.h>
 
-enum RBFTransactionState {
-    RBF_TRANSACTIONSTATE_UNKNOWN,
-    RBF_TRANSACTIONSTATE_REPLACEABLE_BIP125,
-    RBF_TRANSACTIONSTATE_FINAL
+enum class RBFTransactionState {
+    UNKNOWN,
+    REPLACEABLE_BIP125,
+    FINAL
 };
 
-// Check whether the sequence numbers on this transaction are signaling
-// opt-in to replace-by-fee, according to BIP 125
-bool SignalsOptInRBF(const CTransaction &tx);
-
 // Determine whether an in-mempool transaction is signaling opt-in to RBF
 // according to BIP 125
 // This involves checking sequence numbers of the transaction, as well
 // as the sequence numbers of all in-mempool ancestors.
-RBFTransactionState IsRBFOptIn(const CTransaction &tx, CTxMemPool &pool);
+RBFTransactionState IsRBFOptIn(const CTransaction& tx, const CTxMemPool& pool) EXCLUSIVE_LOCKS_REQUIRED(pool.cs);
 
 #endif // BITCOIN_POLICY_RBF_H
