@@ -27,7 +27,7 @@
     </message>
     <message>
         <source>Delete the currently selected address from the list</source>
-        <translation>Kustuta märgistatud aadress loetelust</translation>
+        <translation>Kustuta valitud aadress nimekirjast</translation>
     </message>
     <message>
         <source>Enter address or label to search</source>
@@ -69,10 +69,6 @@
         <source>These are your Bitcoin addresses for sending payments. Always check the amount and the receiving address before sending coins.</source>
         <translation>Need on sinu Bitcoin aadressid maksete saatmiseks. Ennem müntide saatmist kontrolli alati summat ja makse saaja aadressi.</translation>
     </message>
-    <message>
-        <source>These are your Bitcoin addresses for receiving payments. It is recommended to use a new receiving address for each transaction.</source>
-        <translation>Need on sinu Bitcoin aadressid sisenevate maksete vastu võtmiseks. Soovitav on iga tehingu tarbeks kasutada uut aadressi.</translation>
-    </message>
     <message>
         <source>&amp;Copy Address</source>
         <translation>&amp;Kopeeri Aadress</translation>
@@ -106,7 +102,7 @@
     <name>AddressTableModel</name>
     <message>
         <source>Label</source>
-        <translation>Märge</translation>
+        <translation>Silt</translation>
     </message>
     <message>
         <source>Address</source>
@@ -114,7 +110,7 @@
     </message>
     <message>
         <source>(no label)</source>
-        <translation>(märge puudub)</translation>
+        <translation>(silt puudub)</translation>
     </message>
 </context>
 <context>
@@ -125,19 +121,15 @@
     </message>
     <message>
         <source>Enter passphrase</source>
-        <translation>Sisesta salafraas</translation>
+        <translation>Sisesta parool</translation>
     </message>
     <message>
         <source>New passphrase</source>
-        <translation>Uus salafraas</translation>
+        <translation>Uus parool</translation>
     </message>
     <message>
         <source>Repeat new passphrase</source>
-        <translation>Korda salafraasi</translation>
-    </message>
-    <message>
-        <source>Enter the new passphrase to the wallet.&lt;br/&gt;Please use a passphrase of &lt;b&gt;ten or more random characters&lt;/b&gt;, or &lt;b&gt;eight or more words&lt;/b&gt;.</source>
-        <translation>Sisesta uus salafraas rahakotti.&lt;br/&gt;Kasuta salafraasi millles on&lt;b&gt;kümme või rohkem juhuslikku sümbolit&lt;b&gt;,või&lt;b&gt;kaheksa või rohkem sõna&lt;b/&gt;.</translation>
+        <translation>Korda uut parooli</translation>
     </message>
     <message>
         <source>Encrypt wallet</source>
@@ -161,11 +153,7 @@
     </message>
     <message>
         <source>Change passphrase</source>
-        <translation>Vaheta salafraasi</translation>
-    </message>
-    <message>
-        <source>Enter the old passphrase and new passphrase to the wallet.</source>
-        <translation>Sisesta vana salafraas ja uus salafraas rahakotti.</translation>
+        <translation>Muuda parooli</translation>
     </message>
     <message>
         <source>Confirm wallet encryption</source>
@@ -189,11 +177,11 @@
     </message>
     <message>
         <source>Wallet encryption failed due to an internal error. Your wallet was not encrypted.</source>
-        <translation>Rahakoti krüpteerimine ebaõnnestus sisemise tõrke tõttu. Sinu rahakott ei ole krüpteeritud.</translation>
+        <translation>Rahakoti krüpteerimine ebaõnnestus sisemise vea tõttu. Sinu rahakotti ei krüpteeritud.</translation>
     </message>
     <message>
         <source>The supplied passphrases do not match.</source>
-        <translation>Sisestatud salafraasid ei kattu.</translation>
+        <translation>Sisestatud paroolid ei kattu.</translation>
     </message>
     <message>
         <source>Wallet unlock failed</source>
@@ -209,7 +197,7 @@
     </message>
     <message>
         <source>Wallet passphrase was successfully changed.</source>
-        <translation>Rahakoti salafraas on edukalt vahetatud.</translation>
+        <translation>Rahakoti parooli vahetus õnnestus.</translation>
     </message>
     <message>
         <source>Warning: The Caps Lock key is on!</source>
@@ -231,7 +219,7 @@
     </message>
     <message>
         <source>Synchronizing with network...</source>
-        <translation>Võrgusünkimine...</translation>
+        <translation>Võrguga sünkroniseerimine...</translation>
     </message>
     <message>
         <source>&amp;Overview</source>
@@ -255,7 +243,7 @@
     </message>
     <message>
         <source>Quit application</source>
-        <translation>Väljumine</translation>
+        <translation>Välju rakendusest</translation>
     </message>
     <message>
         <source>&amp;About %1</source>
@@ -291,7 +279,7 @@
     </message>
     <message>
         <source>Reindexing blocks on disk...</source>
-        <translation>Kettal olevate blokkide re-indekseerimine...</translation>
+        <translation>Kõvakettal olevate plokkide reindekseerimine...</translation>
     </message>
     <message>
         <source>Send coins to a Bitcoin address</source>
@@ -317,14 +305,6 @@
         <source>&amp;Verify message...</source>
         <translation>&amp;Kontrolli sõnumit...</translation>
     </message>
-    <message>
-        <source>Bitcoin</source>
-        <translation>Bitcoin</translation>
-    </message>
-    <message>
-        <source>Wallet</source>
-        <translation>Rahakott</translation>
-    </message>
     <message>
         <source>&amp;Send</source>
         <translation>&amp;Saada</translation>
@@ -379,7 +359,7 @@
     </message>
     <message>
         <source>&amp;Command-line options</source>
-        <translation>Käsurea valikud</translation>
+        <translation>&amp;Käsurea valikud</translation>
     </message>
     <message numerus="yes">
         <source>%n active connection(s) to Bitcoin network</source>
@@ -399,7 +379,7 @@
     </message>
     <message>
         <source>%1 behind</source>
-        <translation>%1 maas</translation>
+        <translation>%1 ajast maas</translation>
     </message>
     <message>
         <source>Last received block was generated %1 ago.</source>
@@ -407,11 +387,11 @@
     </message>
     <message>
         <source>Transactions after this will not yet be visible.</source>
-        <translation>Peale seda ei ole tehingud veel nähtavad.</translation>
+        <translation>Hilisemad transaktsioonid ei ole veel nähtavad.</translation>
     </message>
     <message>
         <source>Error</source>
-        <translation>Tõrge</translation>
+        <translation>Viga</translation>
     </message>
     <message>
         <source>Warning</source>
@@ -425,6 +405,10 @@
         <source>Up to date</source>
         <translation>Ajakohane</translation>
     </message>
+    <message>
+        <source>&amp;Window</source>
+        <translation>&amp;Aken</translation>
+    </message>
     <message>
         <source>%1 client</source>
         <translation>%1 klient</translation>
@@ -492,7 +476,7 @@
     </message>
     <message>
         <source>Amount:</source>
-        <translation>Summa:</translation>
+        <translation>Kogus</translation>
     </message>
     <message>
         <source>Fee:</source>
@@ -552,11 +536,11 @@
     </message>
     <message>
         <source>Copy amount</source>
-        <translation>Kopeeri summa</translation>
+        <translation>Kopeeri kogus</translation>
     </message>
     <message>
         <source>Copy transaction ID</source>
-        <translation>Kopeeri tehingu ID</translation>
+        <translation>Kopeeri transaktsiooni ID</translation>
     </message>
     <message>
         <source>Copy quantity</source>
@@ -592,13 +576,19 @@
     </message>
     <message>
         <source>(no label)</source>
-        <translation>(märgis puudub)</translation>
+        <translation>(silt puudub)</translation>
     </message>
     <message>
         <source>(change)</source>
         <translation>(vahetusraha)</translation>
     </message>
 </context>
+<context>
+    <name>CreateWalletActivity</name>
+    </context>
+<context>
+    <name>CreateWalletDialog</name>
+    </context>
 <context>
     <name>EditAddressDialog</name>
     <message>
@@ -660,7 +650,7 @@
     <name>Intro</name>
     <message>
         <source>Welcome</source>
-        <translation>Teretulemast</translation>
+        <translation>Tere tulemast</translation>
     </message>
     <message>
         <source>Bitcoin</source>
@@ -668,7 +658,7 @@
     </message>
     <message>
         <source>Error</source>
-        <translation>Tõrge</translation>
+        <translation>Viga</translation>
     </message>
     </context>
 <context>
@@ -705,6 +695,9 @@
         <translation>Vali maksepäringu fail mida avada</translation>
     </message>
 </context>
+<context>
+    <name>OpenWalletActivity</name>
+    </context>
 <context>
     <name>OptionsDialog</name>
     <message>
@@ -715,10 +708,6 @@
         <source>&amp;Main</source>
         <translation>&amp;Peamine</translation>
     </message>
-    <message>
-        <source>MB</source>
-        <translation>MB</translation>
-    </message>
     <message>
         <source>Reset all client options to default.</source>
         <translation>Taasta kõik klientprogrammi seadete vaikeväärtused.</translation>
@@ -825,7 +814,7 @@
     </message>
     <message>
         <source>Error</source>
-        <translation>Tõrge</translation>
+        <translation>Viga</translation>
     </message>
     <message>
         <source>The supplied proxy address is invalid.</source>
@@ -860,7 +849,7 @@
     </message>
     <message>
         <source>Recent transactions</source>
-        <translation>Hiljutised tehingud</translation>
+        <translation>Hiljutised transaktsioonid</translation>
     </message>
     </context>
 <context>
@@ -890,7 +879,7 @@
     </message>
     <message>
         <source>Received</source>
-        <translation>Vastuvõetud</translation>
+        <translation>Vastu võetud</translation>
     </message>
 </context>
 <context>
@@ -948,11 +937,24 @@
         <translation>tundmatu</translation>
     </message>
 </context>
-<context>
-    <name>QObject::QObject</name>
-    </context>
 <context>
     <name>QRImageWidget</name>
+    <message>
+        <source>&amp;Save Image...</source>
+        <translation>&amp;Salvesta Pilt...</translation>
+    </message>
+    <message>
+        <source>&amp;Copy Image</source>
+        <translation>&amp;Kopeeri Pilt</translation>
+    </message>
+    <message>
+        <source>Resulting URI too long, try to reduce the text for label / message.</source>
+        <translation>URI liiga pikk, proovi vähendada märke / sõnumi pikkust.</translation>
+    </message>
+    <message>
+        <source>Error encoding URI into QR Code.</source>
+        <translation>Tõrge URI'st QR koodi loomisel</translation>
+    </message>
     <message>
         <source>Save QR Code</source>
         <translation>Salvesta QR Kood</translation>
@@ -990,7 +992,7 @@
     </message>
     <message>
         <source>Network</source>
-        <translation>Võrgustik</translation>
+        <translation>Võrk</translation>
     </message>
     <message>
         <source>Name</source>
@@ -1002,7 +1004,7 @@
     </message>
     <message>
         <source>Block chain</source>
-        <translation>Ploki jada</translation>
+        <translation>Blokiahel</translation>
     </message>
     <message>
         <source>Current number of blocks</source>
@@ -1018,7 +1020,7 @@
     </message>
     <message>
         <source>Received</source>
-        <translation>Vastuvõetud</translation>
+        <translation>Vastu võetud</translation>
     </message>
     <message>
         <source>Sent</source>
@@ -1044,6 +1046,10 @@
         <source>Services</source>
         <translation>Teenused</translation>
     </message>
+    <message>
+        <source>Ping Time</source>
+        <translation>Pingi Aeg</translation>
+    </message>
     <message>
         <source>Last block time</source>
         <translation>Viimane ploki aeg</translation>
@@ -1056,6 +1062,10 @@
         <source>&amp;Console</source>
         <translation>&amp;Konsool</translation>
     </message>
+    <message>
+        <source>&amp;Network Traffic</source>
+        <translation>&amp;Võrgu Liiklus</translation>
+    </message>
     <message>
         <source>Debug log file</source>
         <translation>Silumise logifail</translation>
@@ -1064,6 +1074,10 @@
         <source>Clear console</source>
         <translation>Puhasta konsool</translation>
     </message>
+    <message>
+        <source>never</source>
+        <translation>mitte kunagi</translation>
+    </message>
     <message>
         <source>Inbound</source>
         <translation>Sisenev</translation>
@@ -1089,7 +1103,7 @@
     <name>ReceiveCoinsDialog</name>
     <message>
         <source>&amp;Amount:</source>
-        <translation>&amp;Summa:</translation>
+        <translation>&amp;Kogus:</translation>
     </message>
     <message>
         <source>&amp;Label:</source>
@@ -1121,7 +1135,7 @@
     </message>
     <message>
         <source>Copy amount</source>
-        <translation>Kopeeri summa</translation>
+        <translation>Kopeeri kogus</translation>
     </message>
 </context>
 <context>
@@ -1134,6 +1148,10 @@
         <source>Copy &amp;Address</source>
         <translation>&amp;Kopeeri Aadress</translation>
     </message>
+    <message>
+        <source>&amp;Save Image...</source>
+        <translation>&amp;Salvesta Pilt...</translation>
+    </message>
     <message>
         <source>Payment information</source>
         <translation>Makse Informatsioon</translation>
@@ -1144,7 +1162,7 @@
     </message>
     <message>
         <source>Amount</source>
-        <translation>Summa</translation>
+        <translation>Kogus</translation>
     </message>
     <message>
         <source>Label</source>
@@ -1158,14 +1176,6 @@
         <source>Wallet</source>
         <translation>Rahakott</translation>
     </message>
-    <message>
-        <source>Resulting URI too long, try to reduce the text for label / message.</source>
-        <translation>URI liiga pikk, proovi vähendada märke / sõnumi pikkust.</translation>
-    </message>
-    <message>
-        <source>Error encoding URI into QR Code.</source>
-        <translation>Tõrge URI'st QR koodi loomisel</translation>
-    </message>
 </context>
 <context>
     <name>RecentRequestsTableModel</name>
@@ -1183,7 +1193,7 @@
     </message>
     <message>
         <source>(no label)</source>
-        <translation>(märge puudub)</translation>
+        <translation>(silt puudub)</translation>
     </message>
     <message>
         <source>(no message)</source>
@@ -1218,7 +1228,7 @@
     </message>
     <message>
         <source>Amount:</source>
-        <translation>Summa:</translation>
+        <translation>Kogus</translation>
     </message>
     <message>
         <source>Fee:</source>
@@ -1290,7 +1300,7 @@
     </message>
     <message>
         <source>Copy amount</source>
-        <translation>Kopeeri summa</translation>
+        <translation>Kopeeri kogus</translation>
     </message>
     <message>
         <source>Copy fee</source>
@@ -1350,7 +1360,7 @@
     </message>
     <message>
         <source>(no label)</source>
-        <translation>(märgis puudub)</translation>
+        <translation>(silt puudub)</translation>
     </message>
 </context>
 <context>
@@ -1454,7 +1464,7 @@
     </message>
     <message>
         <source>Signature</source>
-        <translation>Signatuur</translation>
+        <translation>Allkiri</translation>
     </message>
     <message>
         <source>Copy the current signature to the system clipboard</source>
@@ -1526,11 +1536,11 @@
     </message>
     <message>
         <source>Message signed.</source>
-        <translation>Sõnum allkirjastatud</translation>
+        <translation>Sõnum allkirjastatud.</translation>
     </message>
     <message>
         <source>The signature could not be decoded.</source>
-        <translation>Allkirja polnud võimalik dekodeerida.</translation>
+        <translation>Allkirja ei õnnestunud dekodeerida.</translation>
     </message>
     <message>
         <source>Please check the signature and try again.</source>
@@ -1549,13 +1559,6 @@
         <translation>Sõnum verifitseeritud.</translation>
     </message>
 </context>
-<context>
-    <name>SplashScreen</name>
-    <message>
-        <source>[testnet]</source>
-        <translation>[testnet]</translation>
-    </message>
-</context>
 <context>
     <name>TrafficGraphWidget</name>
     <message>
@@ -1643,7 +1646,7 @@
     </message>
     <message>
         <source>Transaction ID</source>
-        <translation>Tehingu ID</translation>
+        <translation>Transaktsiooni ID</translation>
     </message>
     <message>
         <source>Merchant</source>
@@ -1663,7 +1666,7 @@
     </message>
     <message>
         <source>Amount</source>
-        <translation>Summa</translation>
+        <translation>Kogus</translation>
     </message>
     <message>
         <source>true</source>
@@ -1772,7 +1775,7 @@
     </message>
     <message>
         <source>This month</source>
-        <translation>Käesolev kuu</translation>
+        <translation>Käimasolev kuu</translation>
     </message>
     <message>
         <source>Last month</source>
@@ -1780,7 +1783,7 @@
     </message>
     <message>
         <source>This year</source>
-        <translation>Käesolev aasta</translation>
+        <translation>Käimasolev aasta</translation>
     </message>
     <message>
         <source>Range...</source>
@@ -1824,7 +1827,7 @@
     </message>
     <message>
         <source>Copy transaction ID</source>
-        <translation>Kopeeri tehingu ID</translation>
+        <translation>Kopeeri transaktsiooni ID</translation>
     </message>
     <message>
         <source>Edit label</source>
@@ -1868,7 +1871,7 @@
     </message>
     <message>
         <source>Range:</source>
-        <translation>Ulatus:</translation>
+        <translation>Vahemik:</translation>
     </message>
     <message>
         <source>to</source>
@@ -1878,6 +1881,9 @@
 <context>
     <name>UnitDisplayStatusBarControl</name>
     </context>
+<context>
+    <name>WalletController</name>
+    </context>
 <context>
     <name>WalletFrame</name>
     </context>
@@ -1900,15 +1906,15 @@
     </message>
     <message>
         <source>Backup Wallet</source>
-        <translation>Varundatud Rahakott</translation>
+        <translation>Varunda Rahakott</translation>
     </message>
     <message>
         <source>Wallet Data (*.dat)</source>
-        <translation>Rahakoti andmed (*.dat)</translation>
+        <translation>Rahakoti Andmed (*.dat)</translation>
     </message>
     <message>
         <source>Backup Failed</source>
-        <translation>Varundamine nurjus</translation>
+        <translation>Varundamine Ebaõnnestus</translation>
     </message>
     <message>
         <source>Backup Successful</source>
@@ -1917,10 +1923,6 @@
     </context>
 <context>
     <name>bitcoin-core</name>
-    <message>
-        <source>Bitcoin Core</source>
-        <translation>Bitcoini tuumik</translation>
-    </message>
     <message>
         <source>This is a pre-release test build - use at your own risk - do not use for mining or merchant applications</source>
         <translation>See on test-versioon - kasutamine omal riisikol - ära kasuta mining'uks ega kaupmeeste programmides</translation>
@@ -1949,10 +1951,6 @@
         <source>Error opening block database</source>
         <translation>Tõrge bloki andmebaasi avamisel</translation>
     </message>
-    <message>
-        <source>Error: Disk space is low!</source>
-        <translation>Tõrge: liiga vähe kettaruumi!</translation>
-    </message>
     <message>
         <source>Failed to listen on any port. Use -listen=0 if you want this.</source>
         <translation>Pordi kuulamine nurjus. Soovikorral kasuta -listen=0.</translation>
@@ -1961,10 +1959,6 @@
         <source>Verifying blocks...</source>
         <translation>Kontrollin blokke...</translation>
     </message>
-    <message>
-        <source>Information</source>
-        <translation>Informatsioon</translation>
-    </message>
     <message>
         <source>Signing transaction failed</source>
         <translation>Tehingu allkirjastamine ebaõnnestus</translation>
@@ -1981,10 +1975,6 @@
         <source>Transaction too large</source>
         <translation>Tehing liiga suur</translation>
     </message>
-    <message>
-        <source>Warning</source>
-        <translation>Hoiatus</translation>
-    </message>
     <message>
         <source>Unknown network specified in -onlynet: '%s'</source>
         <translation>Kirjeldatud tundmatu võrgustik -onlynet'is: '%s'</translation>
@@ -2013,9 +2003,5 @@
         <source>Done loading</source>
         <translation>Laetud</translation>
     </message>
-    <message>
-        <source>Error</source>
-        <translation>Tõrge</translation>
-    </message>
 </context>
 </TS>
\ No newline at end of file
