@@ -69,10 +69,6 @@
         <source>These are your Bitcoin addresses for sending payments. Always check the amount and the receiving address before sending coins.</source>
         <translation>Ово су ваше Биткоин адресе за слање уплата. Увек добро проверите износ и адресу на коју шаљете пре него што пошаљете уплату.</translation>
     </message>
-    <message>
-        <source>These are your Bitcoin addresses for receiving payments. It is recommended to use a new receiving address for each transaction.</source>
-        <translation>Ово су ваше Биткоин адресе за примање уплате. Препоручује се да се за сваку трансакцију користи нова адреса.</translation>
-    </message>
     <message>
         <source>&amp;Copy Address</source>
         <translation>&amp;Копирај Адресу</translation>
@@ -135,14 +131,6 @@
         <source>Repeat new passphrase</source>
         <translation>Поновите нову лозинку</translation>
     </message>
-    <message>
-        <source>Show password</source>
-        <translation>Прикажи лозинку</translation>
-    </message>
-    <message>
-        <source>Enter the new passphrase to the wallet.&lt;br/&gt;Please use a passphrase of &lt;b&gt;ten or more random characters&lt;/b&gt;, or &lt;b&gt;eight or more words&lt;/b&gt;.</source>
-        <translation>Унесите нову лозинку за приступ новчанику. &lt;br/&gt;Молимо да користите лозинку од &lt;b&gt;десет или више насумично одабраних карактера, или &lt;b&gt;осам или више речи&lt;/b&gt;.</translation>
-    </message>
     <message>
         <source>Encrypt wallet</source>
         <translation>Шифрирај новчаник</translation>
@@ -167,10 +155,6 @@
         <source>Change passphrase</source>
         <translation>Измену лозинку</translation>
     </message>
-    <message>
-        <source>Enter the old passphrase and new passphrase to the wallet.</source>
-        <translation>Унеси стару лозинку и нову лозинку новчаника.</translation>
-    </message>
     <message>
         <source>Confirm wallet encryption</source>
         <translation>Потврди шифрирање новчаника</translation>
@@ -187,10 +171,6 @@
         <source>Wallet encrypted</source>
         <translation>Новчаник шифриран</translation>
     </message>
-    <message>
-        <source>%1 will close now to finish the encryption process. Remember that encrypting your wallet cannot fully protect your bitcoins from being stolen by malware infecting your computer.</source>
-        <translation>%1 ће се сада затворити како би се завршио процес шифрирања.  Запамтите да шифрирањем свог новчаника не можете у потпуности заштити своје биткоине од крађе од стране злоћудних програма и компјутерских инфекција.</translation>
-    </message>
     <message>
         <source>IMPORTANT: Any previous backups you have made of your wallet file should be replaced with the newly generated, encrypted wallet file. For security reasons, previous backups of the unencrypted wallet file will become useless as soon as you start using the new, encrypted wallet.</source>
         <translation>ВАЖНО: Свакa претходнa резерва новчаника коју сте имали треба да се замени новим, шифрираним фајлом новчаника. Из сигурносних разлога, свака претходна резерва нешифрираног фајла новчаника постаће сувишна, чим почнете да користите нови, шифрирани новчаник.</translation>
@@ -253,10 +233,6 @@
         <source>&amp;Overview</source>
         <translation>&amp;Општи преглед</translation>
     </message>
-    <message>
-        <source>Node</source>
-        <translation>Ноде</translation>
-    </message>
     <message>
         <source>Show general overview of wallet</source>
         <translation>Погледајте општи преглед новчаника</translation>
@@ -313,14 +289,6 @@
         <source>&amp;Change Passphrase...</source>
         <translation>Промени &amp;лозинку...</translation>
     </message>
-    <message>
-        <source>&amp;Sending addresses...</source>
-        <translation>&amp;Адресе за слање...</translation>
-    </message>
-    <message>
-        <source>&amp;Receiving addresses...</source>
-        <translation>&amp;Адресе за пријем</translation>
-    </message>
     <message>
         <source>Open &amp;URI...</source>
         <translation>Отвори &amp;УРИ...</translation>
@@ -329,10 +297,6 @@
         <source>Wallet:</source>
         <translation>Новчаник</translation>
     </message>
-    <message>
-        <source>default wallet</source>
-        <translation>подразумевани новчаник</translation>
-    </message>
     <message>
         <source>Click to disable network activity.</source>
         <translation>Кликни да искључиш активност на мрежи.</translation>
@@ -381,14 +345,6 @@
         <source>&amp;Verify message...</source>
         <translation>&amp;Верификовање поруке...</translation>
     </message>
-    <message>
-        <source>Bitcoin</source>
-        <translation>Биткоин</translation>
-    </message>
-    <message>
-        <source>Wallet</source>
-        <translation>Новчаник</translation>
-    </message>
     <message>
         <source>&amp;Send</source>
         <translation>&amp;Пошаљи</translation>
@@ -501,6 +457,10 @@
         <source>Show the %1 help message to get a list with possible Bitcoin command-line options</source>
         <translation>Прикажи  поруку помоћи %1 за листу са могућим опцијама Биткоин командне линије</translation>
     </message>
+    <message>
+        <source>default wallet</source>
+        <translation>подразумевани новчаник</translation>
+    </message>
     <message>
         <source>%1 client</source>
         <translation>%1 клијент</translation>
@@ -706,6 +666,12 @@
         <translation>(промени)</translation>
     </message>
 </context>
+<context>
+    <name>CreateWalletActivity</name>
+    </context>
+<context>
+    <name>CreateWalletDialog</name>
+    </context>
 <context>
     <name>EditAddressDialog</name>
     <message>
@@ -740,6 +706,10 @@
         <source>Edit sending address</source>
         <translation>Измени адресу за слање</translation>
     </message>
+    <message>
+        <source>The entered address "%1" is not a valid Bitcoin address.</source>
+        <translation>Унета адреса "%1" није важећа Биткоин адреса.</translation>
+    </message>
     <message>
         <source>Could not unlock wallet.</source>
         <translation>Новчаник није могуће откључати.</translation>
@@ -868,6 +838,13 @@
 <context>
     <name>OpenURIDialog</name>
     </context>
+<context>
+    <name>OpenWalletActivity</name>
+    <message>
+        <source>default wallet</source>
+        <translation>подразумевани новчаник</translation>
+    </message>
+    </context>
 <context>
     <name>OptionsDialog</name>
     <message>
@@ -951,9 +928,6 @@
         <translation>nepoznato</translation>
     </message>
 </context>
-<context>
-    <name>QObject::QObject</name>
-    </context>
 <context>
     <name>QRImageWidget</name>
     </context>
@@ -963,10 +937,6 @@
         <source>Last block time</source>
         <translation>Време последњег блока</translation>
     </message>
-    <message>
-        <source>default wallet</source>
-        <translation>подразумевани новчаник</translation>
-    </message>
     <message>
         <source>Yes</source>
         <translation>Da</translation>
@@ -1026,9 +996,13 @@
         <source>Wallet</source>
         <translation>Новчаник</translation>
     </message>
-    </context>
+</context>
 <context>
     <name>RecentRequestsTableModel</name>
+    <message>
+        <source>Date</source>
+        <translation>datum</translation>
+    </message>
     <message>
         <source>Label</source>
         <translation>Налепница</translation>
@@ -1096,6 +1070,10 @@
         <source>Copy fee</source>
         <translation>Копирај провизију</translation>
     </message>
+    <message>
+        <source>Copy after fee</source>
+        <translation>Копирај након провизије</translation>
+    </message>
     <message>
         <source>Copy bytes</source>
         <translation>Копирај бајтове</translation>
@@ -1157,18 +1135,19 @@
         <translation>Alt+П</translation>
     </message>
     </context>
-<context>
-    <name>SplashScreen</name>
-    <message>
-        <source>[testnet]</source>
-        <translation>[testnet]</translation>
-    </message>
-</context>
 <context>
     <name>TrafficGraphWidget</name>
     </context>
 <context>
     <name>TransactionDesc</name>
+    <message>
+        <source>Date</source>
+        <translation>datum</translation>
+    </message>
+    <message>
+        <source>unknown</source>
+        <translation>nepoznato</translation>
+    </message>
     <message>
         <source>Amount</source>
         <translation>Износ</translation>
@@ -1183,6 +1162,10 @@
     </context>
 <context>
     <name>TransactionTableModel</name>
+    <message>
+        <source>Date</source>
+        <translation>datum</translation>
+    </message>
     <message>
         <source>Label</source>
         <translation>Налепница</translation>
@@ -1215,6 +1198,14 @@
         <source>Comma separated file (*.csv)</source>
         <translation>Фајл раздојен тачком (*.csv)</translation>
     </message>
+    <message>
+        <source>Confirmed</source>
+        <translation>Potvrdjen</translation>
+    </message>
+    <message>
+        <source>Date</source>
+        <translation>datum</translation>
+    </message>
     <message>
         <source>Label</source>
         <translation>Налепница</translation>
@@ -1231,12 +1222,23 @@
 <context>
     <name>UnitDisplayStatusBarControl</name>
     </context>
+<context>
+    <name>WalletController</name>
+    </context>
 <context>
     <name>WalletFrame</name>
     </context>
 <context>
     <name>WalletModel</name>
-    </context>
+    <message>
+        <source>Send Coins</source>
+        <translation>Слање новца</translation>
+    </message>
+    <message>
+        <source>default wallet</source>
+        <translation>подразумевани новчаник</translation>
+    </message>
+</context>
 <context>
     <name>WalletView</name>
     <message>
@@ -1250,14 +1252,6 @@
     </context>
 <context>
     <name>bitcoin-core</name>
-    <message>
-        <source>Information</source>
-        <translation>Информације</translation>
-    </message>
-    <message>
-        <source>Warning</source>
-        <translation>Упозорење</translation>
-    </message>
     <message>
         <source>Insufficient funds</source>
         <translation>Nedovoljno sredstava</translation>
@@ -1278,9 +1272,5 @@
         <source>Done loading</source>
         <translation>Završeno učitavanje</translation>
     </message>
-    <message>
-        <source>Error</source>
-        <translation>Greška</translation>
-    </message>
 </context>
 </TS>
\ No newline at end of file
