@@ -65,10 +65,6 @@
         <source>These are your Bitcoin addresses for sending payments. Always check the amount and the receiving address before sending coins.</source>
         <translation>Ovo su Vaše Bitcoin adrese na koju se vrše uplate. Uvek proverite iznos i prijemnu adresu pre slanja novčića.</translation>
     </message>
-    <message>
-        <source>These are your Bitcoin addresses for receiving payments. It is recommended to use a new receiving address for each transaction.</source>
-        <translation>Ovo su Vaše Bitcoin adrese za primanje uplata. Preporučuje se upotreba nove adrese za svaku transakciju.</translation>
-    </message>
     <message>
         <source>&amp;Copy Address</source>
         <translation>&amp;Kopiraj Adresu</translation>
@@ -87,7 +83,7 @@
     </message>
     <message>
         <source>Comma separated file (*.csv)</source>
-        <translation>Zarezom odvojena datoteka (*.csv) </translation>
+        <translation>Zarezom odvojena datoteka (*.csv)</translation>
     </message>
     <message>
         <source>Exporting Failed</source>
@@ -131,10 +127,6 @@
         <source>Repeat new passphrase</source>
         <translation>Ponovo unesite pristupnu frazu</translation>
     </message>
-    <message>
-        <source>Enter the new passphrase to the wallet.&lt;br/&gt;Please use a passphrase of &lt;b&gt;ten or more random characters&lt;/b&gt;, or &lt;b&gt;eight or more words&lt;/b&gt;.</source>
-        <translation>Unesite novu pristupnu frazu u novčanik. &lt;br/&gt;Molimo, koristite pristupnu frazu koja ima &lt;b&gt; deset ili više nasumičnih znakova&lt;/b&gt;, ili &lt;b&gt;osam ili više reči&lt;/b&gt;.</translation>
-    </message>
     <message>
         <source>Encrypt wallet</source>
         <translation>Šifrujte novčanik</translation>
@@ -159,10 +151,6 @@
         <source>Change passphrase</source>
         <translation>Promenite pristupnu frazu</translation>
     </message>
-    <message>
-        <source>Enter the old passphrase and new passphrase to the wallet.</source>
-        <translation>Unesite u novčanik staru pristupnu frazu i novu pristupnu frazu.</translation>
-    </message>
     <message>
         <source>Confirm wallet encryption</source>
         <translation>Potvrdite šifrovanje novčanika</translation>
@@ -179,10 +167,6 @@
         <source>Wallet encrypted</source>
         <translation>Novčanik je šifrovan</translation>
     </message>
-    <message>
-        <source>%1 will close now to finish the encryption process. Remember that encrypting your wallet cannot fully protect your bitcoins from being stolen by malware infecting your computer.</source>
-        <translation>%1 će se ugasiti da bi dovršio proces enkriptovanja. Imajte u vidu da enkripcija novčanika ne garantuje potpunu zaštitu bitcoina od krađe od strane nekog virusa koji bi zarazio vaš kompjuter.</translation>
-    </message>
     <message>
         <source>IMPORTANT: Any previous backups you have made of your wallet file should be replaced with the newly generated, encrypted wallet file. For security reasons, previous backups of the unencrypted wallet file will become useless as soon as you start using the new, encrypted wallet.</source>
         <translation>VAŽNO: Ranije rezervne kopije wallet datoteke trebate zameniti sa novo-kreiranom, enkriptovanom wallet datotekom. Iz sigurnosnih razloga, ranije ne-enkriptovane wallet datoteke će postati neupotrebljive čim počnete koristiti novi, enkriptovani novčanik.</translation>
@@ -245,10 +229,6 @@
         <source>&amp;Overview</source>
         <translation>&amp;Pregled</translation>
     </message>
-    <message>
-        <source>Node</source>
-        <translation>Čvor</translation>
-    </message>
     <message>
         <source>Show general overview of wallet</source>
         <translation>Prikaži opšti pregled novčanika</translation>
@@ -305,14 +285,6 @@
         <source>&amp;Change Passphrase...</source>
         <translation>&amp;Izmeni pristupnu frazu...</translation>
     </message>
-    <message>
-        <source>&amp;Sending addresses...</source>
-        <translation>&amp;Slanje adresa...</translation>
-    </message>
-    <message>
-        <source>&amp;Receiving addresses...</source>
-        <translation>&amp;Primanje adresa...</translation>
-    </message>
     <message>
         <source>Open &amp;URI...</source>
         <translation>Otvori &amp;URI...</translation>
@@ -361,14 +333,6 @@
         <source>&amp;Verify message...</source>
         <translation>&amp;Proveri poruku...</translation>
     </message>
-    <message>
-        <source>Bitcoin</source>
-        <translation>Bitcoin</translation>
-    </message>
-    <message>
-        <source>Wallet</source>
-        <translation>Novčanik</translation>
-    </message>
     <message>
         <source>&amp;Send</source>
         <translation>&amp;Pošalji</translation>
@@ -495,6 +459,12 @@
         <translation>(bez oznake)</translation>
     </message>
     </context>
+<context>
+    <name>CreateWalletActivity</name>
+    </context>
+<context>
+    <name>CreateWalletDialog</name>
+    </context>
 <context>
     <name>EditAddressDialog</name>
     <message>
@@ -533,6 +503,9 @@
 <context>
     <name>OpenURIDialog</name>
     </context>
+<context>
+    <name>OpenWalletActivity</name>
+    </context>
 <context>
     <name>OptionsDialog</name>
     <message>
@@ -560,9 +533,6 @@
         <translation>nepoznato</translation>
     </message>
 </context>
-<context>
-    <name>QObject::QObject</name>
-    </context>
 <context>
     <name>QRImageWidget</name>
     </context>
@@ -578,6 +548,10 @@
         <source>Address</source>
         <translation>Adresa</translation>
     </message>
+    <message>
+        <source>Amount</source>
+        <translation>Kolicina</translation>
+    </message>
     <message>
         <source>Label</source>
         <translation>Oznaka</translation>
@@ -586,9 +560,13 @@
         <source>Wallet</source>
         <translation>Novčanik</translation>
     </message>
-    </context>
+</context>
 <context>
     <name>RecentRequestsTableModel</name>
+    <message>
+        <source>Date</source>
+        <translation>Datum</translation>
+    </message>
     <message>
         <source>Label</source>
         <translation>Oznaka</translation>
@@ -637,20 +615,37 @@
 <context>
     <name>SignVerifyMessageDialog</name>
     </context>
-<context>
-    <name>SplashScreen</name>
-    </context>
 <context>
     <name>TrafficGraphWidget</name>
     </context>
 <context>
     <name>TransactionDesc</name>
+    <message>
+        <source>Date</source>
+        <translation>Datum</translation>
+    </message>
+    <message>
+        <source>unknown</source>
+        <translation>nepoznato</translation>
+    </message>
+    <message>
+        <source>Transaction fee</source>
+        <translation>Taksa transakcije</translation>
+    </message>
+    <message>
+        <source>Amount</source>
+        <translation>Kolicina</translation>
+    </message>
     </context>
 <context>
     <name>TransactionDescDialog</name>
     </context>
 <context>
     <name>TransactionTableModel</name>
+    <message>
+        <source>Date</source>
+        <translation>Datum</translation>
+    </message>
     <message>
         <source>Label</source>
         <translation>Oznaka</translation>
@@ -664,7 +659,11 @@
     <name>TransactionView</name>
     <message>
         <source>Comma separated file (*.csv)</source>
-        <translation>Zarezom odvojena datoteka (*.csv) </translation>
+        <translation>Zarezom odvojena datoteka (*.csv)</translation>
+    </message>
+    <message>
+        <source>Date</source>
+        <translation>Datum</translation>
     </message>
     <message>
         <source>Label</source>
@@ -682,6 +681,9 @@
 <context>
     <name>UnitDisplayStatusBarControl</name>
     </context>
+<context>
+    <name>WalletController</name>
+    </context>
 <context>
     <name>WalletFrame</name>
     </context>
@@ -690,21 +692,17 @@
     </context>
 <context>
     <name>WalletView</name>
+    <message>
+        <source>&amp;Export</source>
+        <translation>&amp;Izvoz</translation>
+    </message>
+    <message>
+        <source>Export the data in the current tab to a file</source>
+        <translation>Izvoz podataka iz trenutne kartice u datoteku</translation>
+    </message>
     </context>
 <context>
     <name>bitcoin-core</name>
-    <message>
-        <source>Bitcoin Core</source>
-        <translation>Bitcoin Core</translation>
-    </message>
-    <message>
-        <source>Information</source>
-        <translation>Informacije</translation>
-    </message>
-    <message>
-        <source>Warning</source>
-        <translation>Upozorenje</translation>
-    </message>
     <message>
         <source>Insufficient funds</source>
         <translation>Nedovoljno sredstava</translation>
@@ -725,9 +723,5 @@
         <source>Done loading</source>
         <translation>Zavrseno ucitavanje</translation>
     </message>
-    <message>
-        <source>Error</source>
-        <translation>Greska</translation>
-    </message>
 </context>
 </TS>
\ No newline at end of file
