@@ -85,10 +85,6 @@
         <source>&amp;Overview</source>
         <translation>&amp;Кўриб чиқиш</translation>
     </message>
-    <message>
-        <source>Node</source>
-        <translation>Улам</translation>
-    </message>
     <message>
         <source>Show general overview of wallet</source>
         <translation>Ҳамённинг умумий кўринишини кўрсатиш</translation>
@@ -133,14 +129,6 @@
         <source>&amp;Change Passphrase...</source>
         <translation>Махфий сўзни &amp;ўзгартириш...</translation>
     </message>
-    <message>
-        <source>&amp;Sending addresses...</source>
-        <translation>&amp;Жўнатилувчи манзиллар...</translation>
-    </message>
-    <message>
-        <source>&amp;Receiving addresses...</source>
-        <translation>&amp;Қабул қилувчи манзиллар...</translation>
-    </message>
     <message>
         <source>Open &amp;URI...</source>
         <translation>Интернет манзилни очиш</translation>
@@ -173,14 +161,6 @@
         <source>&amp;Verify message...</source>
         <translation>Хабарни &amp;тасдиқлаш...</translation>
     </message>
-    <message>
-        <source>Bitcoin</source>
-        <translation>Bitcoin</translation>
-    </message>
-    <message>
-        <source>Wallet</source>
-        <translation>Ҳамён</translation>
-    </message>
     <message>
         <source>&amp;Send</source>
         <translation>&amp;Жўнатиш</translation>
@@ -277,6 +257,14 @@
         <source>Up to date</source>
         <translation>Янгиланган</translation>
     </message>
+    <message>
+        <source>&amp;Window</source>
+        <translation>&amp;Ойна</translation>
+    </message>
+    <message>
+        <source>Minimize</source>
+        <translation>Камайтириш</translation>
+    </message>
     <message>
         <source>Catching up...</source>
         <translation>Банд қилинмоқда...</translation>
@@ -357,6 +345,12 @@
         <translation>Тасдиқланди</translation>
     </message>
     </context>
+<context>
+    <name>CreateWalletActivity</name>
+    </context>
+<context>
+    <name>CreateWalletDialog</name>
+    </context>
 <context>
     <name>EditAddressDialog</name>
     <message>
@@ -475,6 +469,9 @@
         <translation>Тўлов сўрови файлини танлаш</translation>
     </message>
     </context>
+<context>
+    <name>OpenWalletActivity</name>
+    </context>
 <context>
     <name>OptionsDialog</name>
     <message>
@@ -489,10 +486,6 @@
         <source>Size of &amp;database cache</source>
         <translation>&amp;Маълумотлар базаси кеши</translation>
     </message>
-    <message>
-        <source>MB</source>
-        <translation>МБ</translation>
-    </message>
     <message>
         <source>Number of script &amp;verification threads</source>
         <translation>Мавзуларни &amp;тўғрилаш скрипти миқдори</translation>
@@ -722,11 +715,12 @@
         <translation>Номаълум</translation>
     </message>
 </context>
-<context>
-    <name>QObject::QObject</name>
-    </context>
 <context>
     <name>QRImageWidget</name>
+    <message>
+        <source>&amp;Save Image...</source>
+        <translation>Расмни &amp;сақлаш</translation>
+    </message>
     </context>
 <context>
     <name>RPCConsole</name>
@@ -913,10 +907,6 @@
         <source>Requested payments history</source>
         <translation>Сўралган тўлов тарихи</translation>
     </message>
-    <message>
-        <source>&amp;Request payment</source>
-        <translation>Тўловни &amp;сўраш</translation>
-    </message>
     <message>
         <source>Show the selected request (does the same as double clicking an entry)</source>
         <translation>Танланган сўровни кўрсатиш (икки марта босилганда ҳам бир хил амал бажарилсин)</translation>
@@ -948,13 +938,21 @@
         <source>&amp;Save Image...</source>
         <translation>Расмни &amp;сақлаш</translation>
     </message>
+    <message>
+        <source>Amount</source>
+        <translation>Миқдори</translation>
+    </message>
     <message>
         <source>Wallet</source>
         <translation>Ҳамён</translation>
     </message>
-    </context>
+</context>
 <context>
     <name>RecentRequestsTableModel</name>
+    <message>
+        <source>Date</source>
+        <translation>Сана</translation>
+    </message>
     </context>
 <context>
     <name>SendCoinsDialog</name>
@@ -1100,7 +1098,11 @@
     </context>
 <context>
     <name>SendConfirmationDialog</name>
-    </context>
+    <message>
+        <source>Yes</source>
+        <translation>Ҳа</translation>
+    </message>
+</context>
 <context>
     <name>ShutdownWindow</name>
     </context>
@@ -1131,18 +1133,27 @@
         <translation>Барчасини &amp; Тозалаш</translation>
     </message>
     </context>
-<context>
-    <name>SplashScreen</name>
-    <message>
-        <source>[testnet]</source>
-        <translation>[testnet]</translation>
-    </message>
-</context>
 <context>
     <name>TrafficGraphWidget</name>
     </context>
 <context>
     <name>TransactionDesc</name>
+    <message>
+        <source>Date</source>
+        <translation>Сана</translation>
+    </message>
+    <message>
+        <source>unknown</source>
+        <translation>Номаълум</translation>
+    </message>
+    <message>
+        <source>Transaction fee</source>
+        <translation>Ўтказма тўлови</translation>
+    </message>
+    <message>
+        <source>Amount</source>
+        <translation>Миқдори</translation>
+    </message>
     </context>
 <context>
     <name>TransactionDescDialog</name>
@@ -1153,36 +1164,51 @@
     </context>
 <context>
     <name>TransactionTableModel</name>
+    <message>
+        <source>Date</source>
+        <translation>Сана</translation>
+    </message>
     </context>
 <context>
     <name>TransactionView</name>
+    <message>
+        <source>Confirmed</source>
+        <translation>Тасдиқланди</translation>
+    </message>
+    <message>
+        <source>Date</source>
+        <translation>Сана</translation>
+    </message>
     </context>
 <context>
     <name>UnitDisplayStatusBarControl</name>
     </context>
+<context>
+    <name>WalletController</name>
+    </context>
 <context>
     <name>WalletFrame</name>
     </context>
 <context>
     <name>WalletModel</name>
+    <message>
+        <source>Send Coins</source>
+        <translation>Тангаларни жунат</translation>
+    </message>
     </context>
 <context>
     <name>WalletView</name>
+    <message>
+        <source>&amp;Export</source>
+        <translation>&amp;Экспорт</translation>
+    </message>
+    <message>
+        <source>Export the data in the current tab to a file</source>
+        <translation>Жорий ички ойна ичидаги маълумотларни файлга экспорт қилиш</translation>
+    </message>
     </context>
 <context>
     <name>bitcoin-core</name>
-    <message>
-        <source>Bitcoin Core</source>
-        <translation>Bitcoin Core</translation>
-    </message>
-    <message>
-        <source>Information</source>
-        <translation>Маълумот</translation>
-    </message>
-    <message>
-        <source>Warning</source>
-        <translation>Диққат</translation>
-    </message>
     <message>
         <source>Insufficient funds</source>
         <translation>Кам миқдор</translation>
@@ -1203,9 +1229,5 @@
         <source>Done loading</source>
         <translation>Юклаш тайёр</translation>
     </message>
-    <message>
-        <source>Error</source>
-        <translation>Хатолик</translation>
-    </message>
 </context>
 </TS>
\ No newline at end of file
