@@ -5,11 +5,10 @@
 
 #include <script/sigcache.h>
 
-#include <memusage.h>
 #include <pubkey.h>
 #include <random.h>
 #include <uint256.h>
-#include <util.h>
+#include <util/system.h>
 
 #include <cuckoocache.h>
 #include <boost/thread.hpp>
