@@ -1,12 +1,12 @@
-// Copyright (c) 2013-2018 The Bitcoin Core developers
+// Copyright (c) 2013-2019 The Bitcoin Core developers
 // Distributed under the MIT software license, see the accompanying
 // file COPYING or http://www.opensource.org/licenses/mit-license.php.
 
+#include <clientversion.h>
+#include <crypto/siphash.h>
 #include <hash.h>
-#include <utilstrencodings.h>
-#include <test/test_bitcoin.h>
-
-#include <vector>
+#include <util/strencodings.h>
+#include <test/setup_common.h>
 
 #include <boost/test/unit_test.hpp>
 
