@@ -1,4 +1,4 @@
-// Copyright (c) 2016-2017 The Bitcoin Core developers
+// Copyright (c) 2016-2019 The Bitcoin Core developers
 // Distributed under the MIT software license, see the accompanying
 // file COPYING or http://www.opensource.org/licenses/mit-license.php.
 
@@ -12,9 +12,7 @@
 
 #include <support/events.h>
 
-#include <test/test_bitcoin.h>
-
-#include <vector>
+#include <test/setup_common.h>
 
 #include <boost/test/unit_test.hpp>
 
@@ -41,7 +39,7 @@ BOOST_FIXTURE_TEST_SUITE(raii_event_tests, BasicTestingSetup)
 BOOST_AUTO_TEST_CASE(raii_event_creation)
 {
     event_set_mem_functions(tag_malloc, realloc, tag_free);
-    
+
     void* base_ptr = nullptr;
     {
         auto base = obtain_event_base();
@@ -49,7 +47,7 @@ BOOST_AUTO_TEST_CASE(raii_event_creation)
         BOOST_CHECK(tags[base_ptr] == 1);
     }
     BOOST_CHECK(tags[base_ptr] == 0);
-    
+
     void* event_ptr = nullptr;
     {
         auto base = obtain_event_base();
@@ -63,14 +61,14 @@ BOOST_AUTO_TEST_CASE(raii_event_creation)
     }
     BOOST_CHECK(tags[base_ptr] == 0);
     BOOST_CHECK(tags[event_ptr] == 0);
-    
+
     event_set_mem_functions(malloc, realloc, free);
 }
 
 BOOST_AUTO_TEST_CASE(raii_event_order)
 {
     event_set_mem_functions(tag_malloc, realloc, tag_free);
-    
+
     void* base_ptr = nullptr;
     void* event_ptr = nullptr;
     {
