@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) 2017-2018 The Bitcoin Core developers
+# Copyright (c) 2017-2019 The Bitcoin Core developers
 # Distributed under the MIT software license, see the accompanying
 # file COPYING or http://www.opensource.org/licenses/mit-license.php.
 """Test that the wallet can send and receive using all combinations of address types.
@@ -54,15 +54,20 @@ from decimal import Decimal
 import itertools
 
 from test_framework.test_framework import BitcoinTestFramework
+from test_framework.descriptors import (
+    descsum_create,
+    descsum_check,
+)
 from test_framework.util import (
     assert_equal,
     assert_greater_than,
     assert_raises_rpc_error,
-    connect_nodes_bi,
-    sync_blocks,
-    sync_mempools,
+    connect_nodes,
+)
+from test_framework.segwit_addr import (
+    encode,
+    decode,
 )
-
 
 class AddressTypeTest(BitcoinTestFramework):
     def set_test_params(self):
@@ -85,7 +90,7 @@ class AddressTypeTest(BitcoinTestFramework):
         # Fully mesh-connect nodes for faster mempool sync
         for i, j in itertools.product(range(self.num_nodes), repeat=2):
             if i > j:
-                connect_nodes_bi(self.nodes, i, j)
+                connect_nodes(self.nodes[i], j)
         self.sync_all()
 
     def get_balances(self, confirmed=True):
@@ -95,59 +100,122 @@ class AddressTypeTest(BitcoinTestFramework):
         else:
             return [self.nodes[i].getunconfirmedbalance() for i in range(4)]
 
+    # Quick test of python bech32 implementation
+    def test_python_bech32(self, addr):
+        hrp = addr[:4]
+        assert_equal(hrp, "bcrt")
+        (witver, witprog) = decode(hrp, addr)
+        assert_equal(encode(hrp, witver, witprog), addr)
+
     def test_address(self, node, address, multisig, typ):
         """Run sanity checks on an address."""
         info = self.nodes[node].getaddressinfo(address)
-        assert(self.nodes[node].validateaddress(address)['isvalid'])
+        assert self.nodes[node].validateaddress(address)['isvalid']
+        assert_equal(info.get('solvable'), True)
+
         if not multisig and typ == 'legacy':
             # P2PKH
-            assert(not info['isscript'])
-            assert(not info['iswitness'])
-            assert('pubkey' in info)
+            assert not info['isscript']
+            assert not info['iswitness']
+            assert 'pubkey' in info
         elif not multisig and typ == 'p2sh-segwit':
             # P2SH-P2WPKH
-            assert(info['isscript'])
-            assert(not info['iswitness'])
+            assert info['isscript']
+            assert not info['iswitness']
             assert_equal(info['script'], 'witness_v0_keyhash')
-            assert('pubkey' in info)
+            assert 'pubkey' in info
         elif not multisig and typ == 'bech32':
             # P2WPKH
-            assert(not info['isscript'])
-            assert(info['iswitness'])
+            assert not info['isscript']
+            assert info['iswitness']
             assert_equal(info['witness_version'], 0)
             assert_equal(len(info['witness_program']), 40)
-            assert('pubkey' in info)
+            assert 'pubkey' in info
+            self.test_python_bech32(info["address"])
         elif typ == 'legacy':
             # P2SH-multisig
-            assert(info['isscript'])
+            assert info['isscript']
             assert_equal(info['script'], 'multisig')
-            assert(not info['iswitness'])
-            assert('pubkeys' in info)
+            assert not info['iswitness']
+            assert 'pubkeys' in info
         elif typ == 'p2sh-segwit':
             # P2SH-P2WSH-multisig
-            assert(info['isscript'])
+            assert info['isscript']
             assert_equal(info['script'], 'witness_v0_scripthash')
-            assert(not info['iswitness'])
-            assert(info['embedded']['isscript'])
+            assert not info['iswitness']
+            assert info['embedded']['isscript']
             assert_equal(info['embedded']['script'], 'multisig')
-            assert(info['embedded']['iswitness'])
+            assert info['embedded']['iswitness']
             assert_equal(info['embedded']['witness_version'], 0)
             assert_equal(len(info['embedded']['witness_program']), 64)
-            assert('pubkeys' in info['embedded'])
+            assert 'pubkeys' in info['embedded']
         elif typ == 'bech32':
             # P2WSH-multisig
-            assert(info['isscript'])
+            assert info['isscript']
             assert_equal(info['script'], 'multisig')
-            assert(info['iswitness'])
+            assert info['iswitness']
             assert_equal(info['witness_version'], 0)
             assert_equal(len(info['witness_program']), 64)
-            assert('pubkeys' in info)
+            assert 'pubkeys' in info
+            self.test_python_bech32(info["address"])
         else:
             # Unknown type
-            assert(False)
+            assert False
+
+    def test_desc(self, node, address, multisig, typ, utxo):
+        """Run sanity checks on a descriptor reported by getaddressinfo."""
+        info = self.nodes[node].getaddressinfo(address)
+        assert 'desc' in info
+        assert_equal(info['desc'], utxo['desc'])
+        assert self.nodes[node].validateaddress(address)['isvalid']
+
+        # Use a ridiculously roundabout way to find the key origin info through
+        # the PSBT logic. However, this does test consistency between the PSBT reported
+        # fingerprints/paths and the descriptor logic.
+        psbt = self.nodes[node].createpsbt([{'txid':utxo['txid'], 'vout':utxo['vout']}],[{address:0.00010000}])
+        psbt = self.nodes[node].walletprocesspsbt(psbt, False, "ALL", True)
+        decode = self.nodes[node].decodepsbt(psbt['psbt'])
+        key_descs = {}
+        for deriv in decode['inputs'][0]['bip32_derivs']:
+            assert_equal(len(deriv['master_fingerprint']), 8)
+            assert_equal(deriv['path'][0], 'm')
+            key_descs[deriv['pubkey']] = '[' + deriv['master_fingerprint'] + deriv['path'][1:] + ']' + deriv['pubkey']
+
+        # Verify the descriptor checksum against the Python implementation
+        assert descsum_check(info['desc'])
+        # Verify that stripping the checksum and recreating it using Python roundtrips
+        assert info['desc'] == descsum_create(info['desc'][:-9])
+        # Verify that stripping the checksum and feeding it to getdescriptorinfo roundtrips
+        assert info['desc'] == self.nodes[0].getdescriptorinfo(info['desc'][:-9])['descriptor']
+        assert_equal(info['desc'][-8:], self.nodes[0].getdescriptorinfo(info['desc'][:-9])['checksum'])
+        # Verify that keeping the checksum and feeding it to getdescriptorinfo roundtrips
+        assert info['desc'] == self.nodes[0].getdescriptorinfo(info['desc'])['descriptor']
+        assert_equal(info['desc'][-8:], self.nodes[0].getdescriptorinfo(info['desc'])['checksum'])
+
+        if not multisig and typ == 'legacy':
+            # P2PKH
+            assert_equal(info['desc'], descsum_create("pkh(%s)" % key_descs[info['pubkey']]))
+        elif not multisig and typ == 'p2sh-segwit':
+            # P2SH-P2WPKH
+            assert_equal(info['desc'], descsum_create("sh(wpkh(%s))" % key_descs[info['pubkey']]))
+        elif not multisig and typ == 'bech32':
+            # P2WPKH
+            assert_equal(info['desc'], descsum_create("wpkh(%s)" % key_descs[info['pubkey']]))
+        elif typ == 'legacy':
+            # P2SH-multisig
+            assert_equal(info['desc'], descsum_create("sh(multi(2,%s,%s))" % (key_descs[info['pubkeys'][0]], key_descs[info['pubkeys'][1]])))
+        elif typ == 'p2sh-segwit':
+            # P2SH-P2WSH-multisig
+            assert_equal(info['desc'], descsum_create("sh(wsh(multi(2,%s,%s)))" % (key_descs[info['embedded']['pubkeys'][0]], key_descs[info['embedded']['pubkeys'][1]])))
+        elif typ == 'bech32':
+            # P2WSH-multisig
+            assert_equal(info['desc'], descsum_create("wsh(multi(2,%s,%s))" % (key_descs[info['pubkeys'][0]], key_descs[info['pubkeys'][1]])))
+        else:
+            # Unknown type
+            assert False
 
     def test_change_output_type(self, node_sender, destinations, expected_type):
-        txid = self.nodes[node_sender].sendmany(fromaccount="", amounts=dict.fromkeys(destinations, 0.001))
+        txid = self.nodes[node_sender].sendmany(dummy="", amounts=dict.fromkeys(destinations, 0.001))
         raw_tx = self.nodes[node_sender].getrawtransaction(txid)
         tx = self.nodes[node_sender].decoderawtransaction(raw_tx)
 
@@ -166,7 +234,7 @@ class AddressTypeTest(BitcoinTestFramework):
         # Mine 101 blocks on node5 to bring nodes out of IBD and make sure that
         # no coinbases are maturing for the nodes-under-test during the test
         self.nodes[5].generate(101)
-        sync_blocks(self.nodes)
+        self.sync_blocks()
 
         uncompressed_1 = "0496b538e853519c726a2c91e61ec11600ae1390813a627c66fb8be7947be63c52da7589379515d4e0a604f8141781e62294721166bf621e73a82cbf2342c858ee"
         uncompressed_2 = "047211a824f55b505228e4c3d5194c1fcfaa15a456abdf37f9b9d97a4040afc073dee6c89064984f03385237d92167c13e236446b417ab79a0fcae412ae3316b77"
@@ -198,6 +266,7 @@ class AddressTypeTest(BitcoinTestFramework):
             self.log.debug("Old balances are {}".format(old_balances))
             to_send = (old_balances[from_node] / 101).quantize(Decimal("0.00000001"))
             sends = {}
+            addresses = {}
 
             self.log.debug("Prepare sends")
             for n, to_node in enumerate(range(from_node, from_node + 4)):
@@ -228,10 +297,11 @@ class AddressTypeTest(BitcoinTestFramework):
 
                 # Output entry
                 sends[address] = to_send * 10 * (1 + n)
+                addresses[to_node] = (address, typ)
 
             self.log.debug("Sending: {}".format(sends))
             self.nodes[from_node].sendmany("", sends)
-            sync_mempools(self.nodes)
+            self.sync_mempools()
 
             unconf_balances = self.get_balances(False)
             self.log.debug("Check unconfirmed balances: {}".format(unconf_balances))
@@ -242,7 +312,18 @@ class AddressTypeTest(BitcoinTestFramework):
 
             # node5 collects fee and block subsidy to keep accounting simple
             self.nodes[5].generate(1)
-            sync_blocks(self.nodes)
+            self.sync_blocks()
+
+            # Verify that the receiving wallet contains a UTXO with the expected address, and expected descriptor
+            for n, to_node in enumerate(range(from_node, from_node + 4)):
+                to_node %= 4
+                found = False
+                for utxo in self.nodes[to_node].listunspent():
+                    if utxo['address'] == addresses[to_node][0]:
+                        found = True
+                        self.test_desc(to_node, addresses[to_node][0], multisig, addresses[to_node][1], utxo)
+                        break
+                assert found
 
             new_balances = self.get_balances()
             self.log.debug("Check new balances: {}".format(new_balances))
@@ -261,7 +342,7 @@ class AddressTypeTest(BitcoinTestFramework):
         # Fund node 4:
         self.nodes[5].sendtoaddress(self.nodes[4].getnewaddress(), Decimal("1"))
         self.nodes[5].generate(1)
-        sync_blocks(self.nodes)
+        self.sync_blocks()
         assert_equal(self.nodes[4].getbalance(), 1)
 
         self.log.info("Nodes with addresstype=legacy never use a P2WPKH change output")
